import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { AsyncSubject as AsyncSubject$1 } from 'rxjs/AsyncSubject';
import { Inject, InjectionToken, Injector, NgModule, Optional, Pipe } from '@angular/core';
import { CommonModule } from '@angular/common';
import { createStore } from 'redux';

const METADATA_KEY = Symbol('@ReduxState');
const METADATA_DEFAULT = {
    name: '',
    reducers: [],
};
/**
 * @param {?} config
 * @return {?}
 */
function ReduxState(config) {
    return (constructor) => {
        Reflect['defineMetadata'](METADATA_KEY, config, constructor);
        return constructor;
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxStateMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT, Reflect['getMetadata'](METADATA_KEY, target));
}

const METADATA_KEY$2 = Symbol('@ReduxActionController');
const METADATA_DEFAULT$2 = {
    actionPrefix: '',
};
/**
 * @param {?} actionPrefix
 * @return {?}
 */
function ReduxActionController(actionPrefix) {
    return (constructor) => {
        const /** @type {?} */ metadata = { actionPrefix };
        Reflect['defineMetadata'](METADATA_KEY$2, metadata, constructor);
        return constructor;
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxActionControllerMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT$2, Reflect['getMetadata'](METADATA_KEY$2, target));
}

/**
 * @param {?} action
 * @return {?}
 */
function dispatch(action) {
    ReduxRegistry.getStore().then(store => store.dispatch(action));
}
const METADATA_KEY$1 = Symbol('@ReduxActionController');
const METADATA_DEFAULT$1 = {
    methodType: '',
    targetConstructor: null,
};
/**
 * @param {?=} customType
 * @return {?}
 */
function ReduxAction(customType) {
    return (target, propertyKey, descriptor) => {
        const /** @type {?} */ methodType = customType || propertyKey;
        const /** @type {?} */ method = descriptor.value;
        const /** @type {?} */ proxyFunction = function () {
            const /** @type {?} */ metadata = getReduxActionControllerMetadata(target.constructor);
            const /** @type {?} */ type = getActionType(metadata.actionPrefix, methodType);
            const /** @type {?} */ payload = method.apply(this, arguments);
            if (payload instanceof Promise) {
                payload.then((p) => dispatch({ payload: p, type }));
            }
            else {
                dispatch({ payload, type });
            }
            return payload;
        };
        Reflect['defineMetadata'](METADATA_KEY$1, {
            methodType,
            targetConstructor: target.constructor,
        }, proxyFunction);
        descriptor.value = proxyFunction;
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxActionMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT$1, Reflect['getMetadata'](METADATA_KEY$1, target));
}
/**
 * @param {?} type
 * @return {?}
 */
function getActionTypeByFunction(type) {
    const { methodType, targetConstructor } = getReduxActionMetadata(type);
    const { actionPrefix } = getReduxActionControllerMetadata(targetConstructor);
    return getActionType(actionPrefix, methodType);
}
/**
 * @param {...?} rest
 * @return {?}
 */
function getActionType(...rest) {
    return rest.filter((i) => i.length > 0).join('/');
}

const METADATA_KEY$3 = Symbol('@ReduxActionController');
const METADATA_DEFAULT$3 = {
    reducers: [],
};
/**
 * @template S, P
 * @param {?} types
 * @return {?}
 */
function ReduxReducer(types) {
    return (target, propertyKey) => {
        const /** @type {?} */ metadata = getReduxReducerClassMetadata(target.constructor);
        metadata.reducers.push({ types, reducer: target[propertyKey].bind(target) });
        Reflect['defineMetadata'](METADATA_KEY$3, metadata, target.constructor);
        // if (typeof target === 'object') {
        //   target[ propertyKey ].bind(target);
        // }
        return target[propertyKey];
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxReducerClassMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT$3, Reflect['getMetadata'](METADATA_KEY$3, target));
}

class ReduxRegistryReducerItem {
}
class ReduxRegistry {
    /**
     * @param {?} store
     * @return {?}
     */
    static registerStore(store) {
        ReduxRegistry._store.next(store);
        ReduxRegistry._store.complete();
    }
    /**
     * @param {?} stateName
     * @param {?} types
     * @param {?} reducer
     * @return {?}
     */
    static registerReducer(stateName, types, reducer) {
        if (Array.isArray(types)) {
            types.forEach((type) => {
                ReduxRegistry._reducers.push({
                    reducer,
                    stateName,
                    type,
                });
            });
        }
        else {
            ReduxRegistry._reducers.push({
                reducer,
                stateName,
                type: /** @type {?} */ (types),
            });
        }
    }
    /**
     * @param {?} state
     * @return {?}
     */
    static registerState(state) {
        ReduxRegistry.getStore().then((store) => {
            const /** @type {?} */ stateConfig = getReduxStateMetadata(state.constructor);
            Promise.resolve(state.getInitialState()).then((initialState) => {
                stateConfig.reducers
                    .map((reducer) => getReduxReducerClassMetadata(reducer.constructor))
                    .forEach((metadata) => {
                    metadata.reducers.forEach(reducer => {
                        ReduxRegistry.registerReducer(stateConfig.name, reducer.types, reducer.reducer);
                    });
                });
                store.dispatch({
                    payload: {
                        initialState,
                        stateName: stateConfig.name,
                    },
                    type: ReduxRegistry.ACTION_REGISTER_MODULE,
                });
            });
        });
    }
    /**
     * @param {?} type
     * @return {?}
     */
    static getReducerItemsByType(type) {
        return ReduxRegistry._reducers
            .filter(reducerItem => getActionTypeByFunction(reducerItem.type) === type);
    }
    /**
     * @return {?}
     */
    static getStore() {
        return new Promise(ReduxRegistry._store.subscribe.bind(ReduxRegistry._store));
    }
}
ReduxRegistry.ACTION_REGISTER_MODULE = `${ReduxRegistry.name}://registerModule`;
ReduxRegistry._store = new AsyncSubject$1();
ReduxRegistry._reducers = [];

/**
 * @template S
 * @param {?} state
 * @param {?} selector
 * @param {?=} context
 * @return {?}
 */
function selectByState(state, selector, context) {
    if (!selector.startsWith('/')) {
        if (!context) {
            throw new Error('Relative selectors need to have a state context!');
        }
        selector = `/${getReduxStateMetadata(context).name}/${selector}`;
    }
    const /** @type {?} */ selectorArray = selector.split('/');
    return selectorArray
        .filter((propertyKey) => propertyKey !== '')
        .reduce((previousValue, propertyKey) => {
        if (!previousValue || !previousValue[propertyKey]) {
            return null;
        }
        return previousValue[propertyKey];
    }, state);
}
/**
 * @template S
 * @param {?} selector
 * @param {?=} context
 * @return {?}
 */
function select(selector, context) {
    const /** @type {?} */ subject = new BehaviorSubject$1(null);
    ReduxRegistry.getStore().then(store => {
        subject.next(selectByState(store.getState(), selector, context));
        store.subscribe(() => subject.next(selectByState(store.getState(), selector, context)));
    });
    return subject.asObservable();
}

const IS_ROOT_MODULE = new InjectionToken('IS_ROOT_MODULE');
const REDUX_ROOT_MODULE_CONFIG = new InjectionToken('REDUX_ROOT_MODULE_CONFIG');
const REDUX_CHILD_MODULE_CONFIG = new InjectionToken('REDUX_CHILD_MODULE_CONFIG');

class ReduxSelectPipe {
    /**
     * @param {?} moduleConfig
     */
    constructor(moduleConfig) {
        this.moduleConfig = moduleConfig;
    }
    /**
     * @param {?} selector
     * @return {?}
     */
    transform(selector) {
        return select(selector, this.moduleConfig.state);
    }
}
ReduxSelectPipe.decorators = [
    { type: Pipe, args: [{
                name: 'reduxSelect',
            },] },
];
/**
 * @nocollapse
 */
ReduxSelectPipe.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [REDUX_CHILD_MODULE_CONFIG,] },] },
];

/**
 * @param {?} selector
 * @param {?=} context
 * @return {?}
 */
function ReduxSelect(selector, context) {
    return (target, propertyKey) => {
        Object.defineProperty(target, propertyKey, {
            configurable: true,
            enumerable: true,
            value: select(selector, context),
        });
    };
}

const rootReducer = (state, action) => {
    if (action.type === ReduxRegistry.ACTION_REGISTER_MODULE) {
        const { stateName, initialState } = action.payload;
        return Object.assign({}, state, {
            [stateName]: initialState,
        });
    }
    return ReduxRegistry.getReducerItemsByType(action.type)
        .reduce((stateToReduce, reducerItem) => {
        const { stateName, reducer } = reducerItem;
        return Object.assign({}, stateToReduce, {
            [stateName]: reducer(stateToReduce[stateName], action),
        });
    }, state);
};

class ReduxModule {
    /**
     * @param {?=} isRootModule
     * @param {?=} rootConfig
     * @param {?=} childConfig
     * @param {?=} injector
     */
    constructor(isRootModule = false, rootConfig = null, childConfig = null, injector) {
        if (isRootModule) {
            rootConfig = Object.assign({
                store: null,
            }, rootConfig || {});
            ReduxRegistry.registerStore(rootConfig.store || createStore(rootReducer, {}));
        }
        else {
            ReduxRegistry.registerState(injector.get(childConfig.state));
        }
    }
    /**
     * @param {?} config
     * @return {?}
     */
    static forChild(config) {
        return {
            ngModule: ReduxModule,
            providers: [
                config.state,
                { provide: IS_ROOT_MODULE, useValue: false },
                { provide: REDUX_CHILD_MODULE_CONFIG, useValue: config },
            ],
        };
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: IS_ROOT_MODULE, useValue: true },
                { provide: REDUX_ROOT_MODULE_CONFIG, useValue: config },
            ],
        };
    }
}
ReduxModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ReduxSelectPipe,
                ],
                exports: [
                    ReduxSelectPipe,
                ],
                imports: [
                    CommonModule,
                ],
            },] },
];
/**
 * @nocollapse
 */
ReduxModule.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [IS_ROOT_MODULE,] },] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [REDUX_ROOT_MODULE_CONFIG,] },] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [REDUX_CHILD_MODULE_CONFIG,] },] },
    { type: Injector, },
];

/**
 * Generated bundle index. Do not edit.
 */

export { selectByState, select, ReduxSelectPipe, ReduxAction, getReduxActionMetadata, getActionTypeByFunction, getActionType, ReduxActionController, getReduxActionControllerMetadata, ReduxReducer, getReduxReducerClassMetadata, ReduxSelect, ReduxState, getReduxStateMetadata, IS_ROOT_MODULE, REDUX_ROOT_MODULE_CONFIG, REDUX_CHILD_MODULE_CONFIG, ReduxModule, ReduxRegistryReducerItem, ReduxRegistry, rootReducer };
//# sourceMappingURL=ngx-redux-core.js.map
